/*
 * @(#)AdventSasDemo.java
 * Copyright (c) 1997 Advent Network Management, Inc. All Rights Reserved.
 * Please read the associated COPYRIGHTS file for more details.
 * 
 * ADVENT NETWORK MANAGEMENT, INC. MAKES NO REPRESENTATIONS OR WARRANTIES 
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  ADVENT NETWORK 
 * MANAGEMENT, INC. SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY 
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE 
 * OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.util.*;
import java.io.*;
import java.applet.Applet;
import Snmp.*;

public class AdventSasDemo extends Applet {

/** Some of the widgets in the main applet window. */
  TextArea  resultArea;
  TextField host,community,setvalue, currentDir;
  List varbindList,childList,listDir;
  CreateDirDialog dirDialog;
  SaveFileDialog saveDialog;
  AppendFileDialog appendDialog;

  String htmlDir;
    
/** The frame extention class we've created for SNMP operations */
SasClientFunction scFrame;  


/** The fonts to be used */
    Font font = new Font("Helvetica",Font.PLAIN,12);
    Font fontb = new Font("Helvetica",Font.BOLD,12);
    Font fontc = new Font("Courier",Font.PLAIN,12);


/** The applets init method sets up the GUI */
  public void init() {

    setFont(font);
    setBackground(Color.lightGray);

    GridBagLayout gridbag = new GridBagLayout();  // we'll use gridbag
    GridBagConstraints c = new GridBagConstraints();  
    setLayout(gridbag);
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.insets = new Insets(0,0,3,0);
    
    c.gridwidth = 1;
    Label label = new Label("Current Directory");label.setFont(fontb);
    gridbag.setConstraints(label,c);
    add(label);

    c.gridwidth = GridBagConstraints.REMAINDER;
    currentDir = new TextField();
    currentDir.setEditable(false);
    gridbag.setConstraints(currentDir,c);
    add(currentDir);


    c.gridwidth = 1;
    label = new Label("Directories");label.setFont(fontb);
    gridbag.setConstraints(label,c);
    add(label);

    c.gridwidth = 1;
    label = new Label("Files");label.setFont(fontb);
    gridbag.setConstraints(label,c);
    add(label);

    c.gridwidth = GridBagConstraints.REMAINDER;
    label = new Label("Debug Messages");label.setFont(fontb);
    gridbag.setConstraints(label,c);
    add(label);


    	
    c.weighty = 1.0;
    c.gridheight = 12;
    c.gridwidth = 1;
    listDir = new List(12, false);
    listDir.setFont(fontb);listDir.setBackground(Color.lightGray);
    gridbag.setConstraints(listDir,c);
    add(listDir);

        
    c.weighty = 1.0;
    c.gridheight = 12;
    c.gridwidth = 1;
    childList = new List(12, false);
    childList.setFont(fontb);childList.setBackground(Color.lightGray);
    gridbag.setConstraints(childList,c);
    add(childList);

 
    c.gridwidth = GridBagConstraints.REMAINDER;
    resultArea = new TextArea("\n",21,50);
    resultArea.setEditable(false);
    resultArea.setFont(fontc);
    gridbag.setConstraints(resultArea,c);
    add(resultArea);

            
	// Bottom panel is a panel sub-class we've defined for a row of buttons
    c.weighty = 0.0;
    c.gridheight = 1;
    //String buttons[] = {"SaveFile", "AppendFile", "DeleteFile", "CreateDir", "DeleteDir", "ListDir"};
      String buttons[] = {"CreateDir", "DeleteDir", "SaveFile", "AppendFile", "DeleteFile"};
    ButtonPanel bp = new ButtonPanel(buttons);
    bp.setFont(fontb);
    gridbag.setConstraints(bp,c);
    add(bp);

    scFrame = new SasClientFunction(this);
    htmlDir = getPrevDir(this.getDocumentBase().getFile());
    
    scFrame.listFiles(htmlDir);

     	// We need a dialog for create directory
	// We'll use the CreatDirDialog class - although it's really a frame.
    dirDialog = new CreateDirDialog("Create Directory",true,"Enter Directory Name","",30);
    dirDialog.sframe = scFrame;  	// pass the snmpFrame object to the dialog
				// since it needs to call the createDir method 

    saveDialog = new SaveFileDialog();
    saveDialog.sframe = scFrame;  	// pass the snmpFrame object to the dialog
				// since it needs to call the saveFile method 

    appendDialog = new AppendFileDialog();
    appendDialog.sframe = scFrame;  	// pass the snmpFrame object to the dialog
				// since it needs to call the saveFile method 


  } // end of constructor


/** Actions for each of the buttons, choice, and list */
  public boolean action(Event e, Object arg) {

	if (e.target instanceof Button) {   // for each button call a method
          
            if (arg.equals("Get File")) scFrame.getFile();
            else if (arg.equals("SaveFile")){saveDialog.show();}
            else if (arg.equals("AppendFile")){appendDialog.show();}
            else if (arg.equals("DeleteFile")) scFrame.deleteFile(childList.getSelectedItem());
            else if (arg.equals("CreateDir"))  dirDialog.show();
            else if (arg.equals("DeleteDir")) scFrame.deleteDir();
            else if (arg.equals("Clear")) resultArea.setText("");
            else if (arg.equals("ListDir")) scFrame.listFiles();
            else System.err.println("Where did this come from: Button: " +arg);

	}
	else if (e.target.equals(listDir)) { // sub-node selected
	   scFrame.downTree((String) arg);
	}
    
	return true;
    }


    /** The stop method, used when applet is no longer on screen */
    public void start() {
        //scFrame.listFiles(".");
    }

    String getPrevDir(String dir) {
    if(dir.endsWith("/"))
        dir = dir.substring(0,dir.length()-1);
    int index = dir.lastIndexOf("/");

    return (dir.substring(0,index+1));

  }


    /** The stop method, used when applet is no longer on screen */
    public void stop() {
//      System.err.println("Stopping MIB Browser Applet");
    }

    /** The destroy method needed to stop any threads, etc. */
    public void destroy() {
      System.err.println("Destroying All MIB Browser Threads");
      if (scFrame == null) return;
      scFrame.api.stop();

	// close each session
      for (Enumeration e = scFrame.api.sessionList.elements();e.hasMoreElements();) 
        ((SnmpSession)e.nextElement()).close();
      
    }

/** To provide applet information */
    public String getAppletInfo() {
	return("SAS Demo from Advent Network Management, Version 1.2");
    }

}



