/*
 * @(#)SasClientFunction.java
 * Copyright (c) 1997 Advent Network Management, Inc. All Rights Reserved.
 * Please read the associated COPYRIGHTS file for more details.
 * 
 * ADVENT NETWORK MANAGEMENT, INC. MAKES NO REPRESENTATIONS OR WARRANTIES 
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  ADVENT NETWORK 
 * MANAGEMENT, INC. SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY 
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE 
 * OR ITS DERIVATIVES.
 */


import java.applet.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;

import Snmp.*;

public class SasClientFunction {
	// This help/debug window only has a text area and a close button.
  TextArea text = null;
  Button close = null;

	// We need a reference to our applet to get some data
  AdventSasDemo sasBrowser = null;

	// The fonts used
  Font fontb;
  Font font;
	// The window title
  String advent = new String("ADVENT MIB Browser Help/Debug Output");
  
	// This is the MIB module currently in use - chosen in GUI
  MibModule currentModule;

  SnmpAPI api;
  SnmpSession session;
 
/** The constructor takes the applet as an argument. 
 *  It sets up the window and calls initSnmp()
 */
  public SasClientFunction(AdventSasDemo  app) { 

    sasBrowser = (AdventSasDemo) app;
    initSnmp();
  } // end of constructor


 void initSnmp() {
    api = new SnmpAPI();
    api.start();
    session = new SnmpSession(api);
    try {
        session.open(sasBrowser);
    }
    catch (Exception e) {
        System.out.println("Unable to open Session");
    }
 }

/** If a sub-node is selected, we go down the MIB tree */
  void downTree(String child) {
  
    String dir;
    if(child.equals(".." + File.separator))
        dir = sasBrowser.getPrevDir(sasBrowser.currentDir.getText());
    else
        dir = new String(sasBrowser.currentDir.getText() + child + "/");
        //dir = new String(sasBrowser.currentDir.getText() + child + File.separator);
    if(dir.equals(""))dir = dir + "/";
    listFiles(dir);


  } // end of downTree()


  void println(String s) {
    sasBrowser.resultArea.appendText(s + "\n"); 
  }


/** For Demonstrating SAS functionalities */

    Frame fra = new Frame("Choose the file you want to save in the Webserver");

    void saveFile(String saveText, String fileName)
    {
        try {
            byte data[] = new byte[saveText.length()];
            saveText.getBytes(0,saveText.length(),data,0);
                if(session.sasclient != null)
                    session.sasclient.saveFile(fileName,data);
                else
                    System.out.println("not connected to sas");
        }
        catch (Exception e){
            System.out.println(e);
            e.printStackTrace();
        }
    }

    void appendFile(String appendText, String fileName)
    {
        
        byte data[] = new byte[appendText.length()];
        try {
            appendText.getBytes(0,appendText.length(),data,0);
        
            if(session.sasclient != null)
                session.sasclient.appendFile(fileName,data);
            else
                System.out.println("not connected to sas");
        }
        catch (Exception e){
            System.out.println(e);
            e.printStackTrace();
        }

    }

    void deleteFile(String fileName)
    {
        if(fileName == null) {
            println("no file selected");
            return;
        }
           try {
            if(session.sasclient != null)
                session.sasclient.deleteFile(fileName);
            else
                System.out.println("not connected to sas");
        }
        catch (Exception e){
            System.out.println(e);
            e.printStackTrace();
        }


    }

    void createDir(String dirName)
    {
        try {
            if(session.sasclient != null)
                session.sasclient.createDir(dirName);
            else
            {
                println("not connected to SAS");
                return;
            }
            listFiles(sasBrowser.currentDir.getText());

         }
         catch(Exception e){e.printStackTrace();}
        
    }

    void deleteDir()
    {
        String dirName = sasBrowser.listDir.getSelectedItem();
        if(dirName == null) {
            println("No directory selected");
            return;
        }
            
        try {
            if(session.sasclient != null)
                session.sasclient.deleteDir(dirName);
            else
            {
                println("not connected to SAS");
                return;
            }
            listFiles(sasBrowser.currentDir.getText());

         }
         catch(Exception e){e.printStackTrace();}

    }

    // some constants for clientCall types
    static int LIST_DIR_REQ = 1;
    static int LIST_DIR_RESP = 2;
    static int GET_FILE_REQ = 3;
    static int GET_FILE_RESP = 4;

    void listFiles(String dir)
    {
        byte data[] = null;
        String listedFiles = null;

        String modDir;
        int i = dir.indexOf("|");
        if(i>0)modDir = dir.substring(i+1);
        else modDir = dir;

        try {
            ByteArrayOutputStream baO = new ByteArrayOutputStream();
            DataOutputStream dO = new DataOutputStream(baO);
            dO.writeInt(LIST_DIR_REQ);
            dO.writeUTF(modDir);

            data = baO.toByteArray();

            if(session.sasclient != null)
                data = session.sasclient.clientCall(data);
            else
            {
                println("Not connected to SAS server");
                return;
            }

        DataInputStream dI = new DataInputStream(new ByteArrayInputStream(data));

        int type = dI.readInt();

        if(type == LIST_DIR_RESP)
            listedFiles = dI.readUTF();

        }
        catch (Exception e) {
            System.out.println(e);
            e.printStackTrace();
        }

        StringTokenizer toks = new StringTokenizer(listedFiles,"\n");
        int num = sasBrowser.childList.countItems();
    if (num>0) sasBrowser.childList.delItems(0,num-1);
     num = sasBrowser.listDir.countItems();
    if (num>0) sasBrowser.listDir.delItems(0,num-1);

    //To handle netscape when loaded from classpath
    boolean netscapeRootDir = false;
    if(dir.length() == 4)
    {
        if(dir.indexOf("|/") > 0)
            netscapeRootDir = true;
    }

    if(!((dir.equals("/")) || (dir.equals("\\")) || (netscapeRootDir)))
        sasBrowser.listDir.addItem(".." + File.separator);

    while(toks.hasMoreTokens())
    {
        String file = toks.nextToken();
        if(file.indexOf("<DIR>") < 0)
  	        sasBrowser.childList.addItem(file);
        else
        {
            file = file.substring(5);
            sasBrowser.listDir.addItem(file);
        }
    }
    sasBrowser.currentDir.setText(dir);
  }


/*  void listFiles(String dir)
    {
        byte data[] = null;
        String listedFiles = null;

        try {
            ByteArrayOutputStream baO = new ByteArrayOutputStream();
            DataOutputStream dO = new DataOutputStream(baO);
            dO.writeInt(LIST_DIR_REQ);
            if(dir.equals(""))
                dO.writeUTF(".");
            else
                dO.writeUTF(dir);

            data = baO.toByteArray();

            if(session.sasclient != null)
                data = session.sasclient.clientCall(data);
            else
            {
                println("not connected to sas");
                return;
            }

        DataInputStream dI = new DataInputStream(new ByteArrayInputStream(data));

        int type = dI.readInt();

        if(type == LIST_DIR_RESP)
            listedFiles = dI.readUTF();

        }
        catch (Exception e) {
            System.out.println(e);
            e.printStackTrace();
        }

        StringTokenizer toks = new StringTokenizer(listedFiles,"\n");
        int num = sasBrowser.childList.countItems();
    if (num>0) sasBrowser.childList.delItems(0,num-1);
     num = sasBrowser.listDir.countItems();
    if (num>0) sasBrowser.listDir.delItems(0,num-1);

    sasBrowser.listDir.addItem(".." + File.separator);
    while(toks.hasMoreTokens())
    {
        String file = toks.nextToken();
        if(file.indexOf("<DIR>") < 0)
  	        sasBrowser.childList.addItem(file);
        else
        {
            file = file.substring(5);
            sasBrowser.listDir.addItem(file);
        }
    }
    sasBrowser.currentDir.setText(dir);

  }
*/

  void listFiles()
  {
  }


  void getFile()
  {
       String file = new String(sasBrowser.currentDir.getText() + File.separator + sasBrowser.childList.getSelectedItem());
       byte[] data = null;
       String gotFile = null;

            try {
            ByteArrayOutputStream baO = new ByteArrayOutputStream();
            DataOutputStream dO = new DataOutputStream(baO);
            dO.writeInt(GET_FILE_REQ);
            dO.writeUTF(file);

            data = baO.toByteArray();

            if(session.sasclient != null)
                data = session.sasclient.clientCall(data);
            else
            {
                println("not connected to sas");
                return;
            }

        DataInputStream dI = new DataInputStream(new ByteArrayInputStream(data));

        int type = dI.readInt();

        if(type == GET_FILE_RESP)
            gotFile = dI.readUTF();
        else
        {
            println("Not able to get the file");
            return;
        }
        FileOutputStream fos = new FileOutputStream(sasBrowser.childList.getSelectedItem() +".sas");
        dO = new DataOutputStream(fos);
        dO.writeBytes(gotFile);
        println("Successfully got the file");

        }
        catch (Exception e) {
            System.out.println(e);
            e.printStackTrace();
        }


  }

}