/*
 * @(#)snmpgetnext.java
 * Copyright (c) 1996 Advent Network Management, Inc. All Rights Reserved.
 * Please read the associated COPYRIGHTS file for more details.
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation without fee is hereby granted provided 
 * that this copyright notice appears in all copies. 
 *
 * ADVENT NETWORK MANAGEMENT, INC. MAKES NO REPRESENTATIONS OR WARRANTIES 
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  ADVENT NETWORK 
 * MANAGEMENT, INC. SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY 
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE 
 * OR ITS DERIVATIVES.
 */

import java.lang.*;
import java.util.*;
import java.net.*;
import Snmp.*;
import ParseOptions;

public class snmpgetnext {

  public static void main(String args[]) {
    SnmpAPI api;
			// Take care of getting options
    String usage = "snmpgetnext [-d] [-c community] [-m MIB_file] [-m2 Second_MIB_file] [-p port] [-t timeout] [-r retries] host OID [OID] ..."; 
    String options[] = { "-d", "-c", "-m", "-m2", "-p", "-r", "-t" };
    String values[] = { "None", null,null, null, null, null, null};
      
    ParseOptions opt = new ParseOptions(args,options,values, usage);

                 	// Start SNMP API
    api = new SnmpAPI();
    api.start();
    if (values[0].equals("Set")) api.DEBUG = true;

			// Next deal with MIBs
    MibModule m1, m2;
    if ( (values[2] !=null)||(values[3] !=null) ) try { // Open the MIB module 

	System.out.println("Loading MIB files: " + values[2] +" "+ values[3]);
	if (values[2] != null) m1 = new MibModule(values[2], api,api.DEBUG);
	if (values[3] != null) m2 = new MibModule(values[3], api,api.DEBUG);
	System.out.println("Done.");

    } catch(Exception e) { System.err.println(e); }


    if (opt.remArgs.length<2) opt.usage_error();

           	// Open session and set remote host & port if needed, 
    SnmpSession session = new SnmpSession(api);

    session.peername = opt.remArgs[0];
    if (values[1] != null) session.community = values[1];

    try { 
      if (values[4] != null) session.remote_port = Integer.parseInt(values[4]);
      if (values[5] != null) session.retries = Integer.parseInt(values[5]);
      if (values[6] != null) session.timeout = Integer.parseInt(values[6]);
    } 
    catch (NumberFormatException ex) { System.err.println("Invalid Integer Arg"); }


		// Build get request PDU
    SnmpPDU pdu = new SnmpPDU(api);
    pdu.command = api.GETNEXT_REQ_MSG;

    for (int i=1;i<opt.remArgs.length;i++) { // add OIDs
      SnmpOID oid = new SnmpOID(opt.remArgs[i], api);
      if (oid.toValue() == null) System.err.println("Invalid OID argument: " + opt.remArgs[i]);
      else pdu.addNull(oid);      
    }

    try { 
	session.open(); 
		// Send PDU
     	pdu = session.syncSend(pdu); 
    } catch (SnmpException e) {
	System.err.println("Sending PDU: "+e.getMessage());
	System.exit(1);
    }

    if (pdu == null) {  // timeout
          System.out.println("Request timed out to: " + opt.remArgs[0] );
          System.exit(1);
    }

		// print and exit
    if (api.DEBUG) System.out.println("Response received from " +pdu.address+
                         ", community: " + pdu.community);
    if (pdu.errstat != 0) 
         System.out.println("Error Indication in response: " +
                            SnmpException.exceptionString((byte)pdu.errstat) + 
			    "\nErrindex: " + pdu.errindex);
    System.out.println(pdu.printVarBinds());

    System.exit(0);

  }

}