/*
 * @(#)snmptrap.java
 * Copyright (c) 1996 Advent Network Management, Inc. All Rights Reserved.
 * Please read the associated COPYRIGHTS file for more details.
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation without fee is hereby granted provided 
 * that this copyright notice appears in all copies. 
 *
 * ADVENT NETWORK MANAGEMENT, INC. MAKES NO REPRESENTATIONS OR WARRANTIES 
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  ADVENT NETWORK 
 * MANAGEMENT, INC. SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY 
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE 
 * OR ITS DERIVATIVES.
 */

import java.lang.*;
import java.util.*;
import java.net.*;
import Snmp.*;
import ParseOptions;

public class snmptrap {

  public static void main(String args[]) {
    SnmpAPI api;
			// Take care of getting options
    String usage = "snmptrap [-d] [-c community] [-m MIB_file] [-m2 Second_MIB_file] [-p port] \nhost enterprise agent-addr generic-trap specific-trap timeticks \n[OID {INTEGER | STRING | GAUGE | TIMETICKS | OPAQUE | IPADDRESS | COUNTER} value] ..."; 
    String options[] = { "-d", "-c", "-m", "-m2", "-p"};
    String values[] = { "None", null,null, null, null};
      
    ParseOptions opt = new ParseOptions(args,options,values, usage);
    if (opt.remArgs.length<4) opt.usage_error();

                 	// Start SNMP API
    api = new SnmpAPI();
    api.start();
    if (values[0].equals("Set")) api.DEBUG = true;

			// Next deal with MIBs
    MibModule m1, m2;
    if ( (values[2] !=null)||(values[3] !=null) ) try { // Open the MIB module 

	System.out.println("Loading MIB files: " + values[2] +" "+ values[3]);
	if (values[2] != null) m1 = new MibModule(values[2], api,api.DEBUG);
	if (values[3] != null) m2 = new MibModule(values[3], api,api.DEBUG);
	System.out.println("Done.");

    } catch(Exception e) { System.err.println(e); }


           	// Open session and set remote host & port if needed, 
    SnmpSession session = new SnmpSession(api);

    session.peername = opt.remArgs[0];
    if (values[1] != null) session.community = values[1];

    try {
     if (values[4] != null) session.remote_port = Integer.parseInt(values[4]);
     else session.remote_port = 162;
    } catch (NumberFormatException ex) {
      System.err.println("Invalid Integer Arg");
    }


		// Build trap request PDU
    SnmpPDU pdu = new SnmpPDU(api);
    pdu.command = api.TRP_REQ_MSG;

    try { // fill in reqd trap PDU fields
      pdu.enterprise = new SnmpOID(opt.remArgs[1]);
      pdu.agent_addr = InetAddress.getByName(opt.remArgs[2]);
      pdu.trap_type = Integer.parseInt(opt.remArgs[3]);
      if (opt.remArgs.length>4) 
	pdu.specific_type = Integer.parseInt(opt.remArgs[4]);
      if (opt.remArgs.length>5) 
	      pdu.time = Integer.parseInt(opt.remArgs[5]);
    } catch (Exception ex) { 
      System.err.println("error in one or more required fields: "+ex);
      opt.usage_error();
    }

    for (int i=6;i<opt.remArgs.length;) { // add Variable Bindings

      if (opt.remArgs.length < i+3) opt.usage_error(); //need "{OID type value}"
      SnmpOID oid = new SnmpOID(opt.remArgs[i++], api);

      if (oid.toValue() == null) System.err.println("Invalid OID argument: " + opt.remArgs[i]);
      else addVarBind(api,pdu, oid, opt.remArgs[i++], opt.remArgs[i++]);

    } // end of add variable bindings

    try { 
	session.open(); 
		// Send PDU
     	session.send(pdu); 
    } catch (SnmpException e) {
	System.err.println("Sending PDU"+e.getMessage());
    }

    System.exit(0);

  }


/** adds the varbind  with specified oid, type and value to the pdu */
  static void addVarBind(SnmpAPI api, SnmpPDU pdu, SnmpOID oid, 
				String type, String value) {

      LeafSyntax syntax = null;
      if (type.equals("INTEGER")) {
	syntax = new LeafSyntax("INTEGER", SnmpAPI.INTEGER);
      } else if (type.equals("STRING")) {
	syntax = new LeafSyntax("OCTET STRING", SnmpAPI.STRING);
      } else if (type.equals("GAUGE")) {
	syntax = new LeafSyntax("Gauge", SnmpAPI.GAUGE);
      } else if (type.equals("TIMETICKS")) {
	syntax = new LeafSyntax("TimeTicks", SnmpAPI.TIMETICKS);
      } else if (type.equals("OPAQUE")) {
	syntax = new LeafSyntax("Opaque", SnmpAPI.OPAQUE);
      } else if (type.equals("IPADDRESS")) {
	syntax = new LeafSyntax("IpAddress", SnmpAPI.IPADDRESS);
      } else if (type.equals("COUNTER")) {
	syntax = new LeafSyntax("Counter", SnmpAPI.COUNTER);
      } else if (type.equals("OID")) { 
	syntax = new LeafSyntax("OBJECT IDENTIFIER", SnmpAPI.OBJID);
      } else { // unknown type
	System.err.println("Invalid variable type: " + type);
	return;
      }

      if (oid.node != null) { // if we have a MIB module node

	if (oid.node.syntax == null) { // if node is not a leaf node	
	  System.err.println("Invalid MIB node - not leaf node: " + oid);
	  return;  
	} else { // if a leaf node
	  if (syntax.type != oid.node.syntax.type) { // if types don't match
	    System.err.println("MIB and variable type mismatch: " 
				+ oid.node.syntax.type + type);
	    return;  
	  }	  // end if types don't match 
	} // end if a leaf node
      } // end if we have a MIB module

      SnmpVarBind varbind = new SnmpVarBind();
      varbind.oid = oid;
	
		// Use LeafSyntax method to create variable
    try { varbind.variable = syntax.createVariable(value); }
    catch (SnmpException e) {
	System.err.println("Cannot create variable: " +oid+" with value: "+value);
     	return;
    } 

    pdu.variables.addElement(varbind);

  } // end of addVarBind()

}

