/*
 * @(#)snmptrapd.java
 * Copyright (c) 1996 Advent Network Management, Inc. All Rights Reserved.
 * Please read the associated COPYRIGHTS file for more details.
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation without fee is hereby granted provided 
 * that this copyright notice appears in all copies. 
 *
 * ADVENT NETWORK MANAGEMENT, INC. MAKES NO REPRESENTATIONS OR WARRANTIES 
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  ADVENT NETWORK 
 * MANAGEMENT, INC. SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY 
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE 
 * OR ITS DERIVATIVES.
 */

import java.lang.*;
import java.util.*;
import java.net.*;
import Snmp.*;
import ParseOptions;

public class snmptrapd {

  public static void main(String args[]) {
    SnmpAPI api;
			// Take care of getting options
    String usage = "snmptrapd [-d] [-c community] [-m MIB_file] [-m2 Second_MIB_file] [-p port]";
    String options[] = { "-d", "-c", "-m", "-m2", "-p"};
    String values[] = { "None", null,null, null, null};
      
    ParseOptions opt = new ParseOptions(args,options,values, usage);

                 	// Start SNMP API
    api = new SnmpAPI();
    api.start();
    if (values[0].equals("Set")) api.DEBUG = true;

			// Next deal with MIBs
    MibModule m1, m2;
    if ( (values[2] !=null)||(values[3] !=null) ) try { // Open the MIB module 

	System.out.println("Loading MIB files: " + values[2] +" "+ values[3]);
	if (values[2] != null) m1 = new MibModule(values[2], api,api.DEBUG);
	if (values[3] != null) m2 = new MibModule(values[3], api,api.DEBUG);
	System.out.println("Done.");

    } catch(Exception e) { System.err.println(e); }


    if (opt.remArgs.length>0) opt.usage_error();

           	// Open session and set local port
    SnmpSession session = new SnmpSession(api);

    if (values[1] != null) session.community = values[1];
    else session.community = null;

    try {  // use specified or standard SNMP trap port
      if (values[4] != null) session.local_port = Integer.parseInt(values[4]);
      else session.local_port = api.SNMP_TRAP_PORT;
    }
    catch (NumberFormatException ex) { System.err.println("Invalid Integer Arg"); }

	// Open the session
    try { session.open(); }
    catch (SnmpException e) { 
      System.err.println(e); 
      System.exit(1);
    }

    SnmpPDU pdu;
    while (true) {  // listen forever
      while ((pdu = session.receive(0)) == null) ;

      if (pdu.command != api.TRP_REQ_MSG) 
	System.err.println("Non trap PDU received, printing anyway.");
	
      System.out.println("Trap received from: "
			  +pdu.address+", community: " + pdu.community);

      pdu.enterprise.getNode(api); // See if we have the MIB node
      System.out.println("Enterprise: " + pdu.enterprise);

      System.out.println("Agent: " + pdu.agent_addr);

      System.out.println("TRAP_TYPE: " + pdu.trap_type);
      System.out.println("SPECIFIC NUMBER: " + pdu.specific_type);

      System.out.println("Time: " + pdu.time+"\nVARBINDS:");

      for (Enumeration e = pdu.variables.elements() ; e.hasMoreElements() ;)  
        System.out.println(((SnmpVarBind) e.nextElement()).toTagString());
       
      System.out.println(""); // a blank line between traps

    } // end of listen forever

  } // end of main

}

