/*
 * @(#)snmpgetnext.java
 * Copyright (c) 1996 Advent Network Management, Inc. All Rights Reserved.
 * Please read the associated COPYRIGHTS file for more details.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation without fee is hereby granted provided
 * that this copyright notice appears in all copies.
 *
 * ADVENT NETWORK MANAGEMENT, INC. MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  ADVENT NETWORK
 * MANAGEMENT, INC. SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
 * OR ITS DERIVATIVES.
 */

import java.lang.*;
import java.util.*;
import java.net.*;
import Snmp.*;
import ParseOptions;

public class snmpwalk {

  public static void main(String args[]) {
    SnmpAPI api;
			// Take care of getting options
    String usage = "snmpwalk [-d] [-c community] [-m MIB_file] [-m2 Second_MIB_file] [-p port] [-t timeout] [-r retries] host OID";
    String options[] = { "-d", "-c", "-m", "-m2", "-p", "-r", "-t" };
    String values[] = { "None", null,null, null, null, null, null};

    ParseOptions opt = new ParseOptions(args,options,values, usage);

                 	// Start SNMP API
    api = new SnmpAPI();
    api.start();
    if (values[0].equals("Set")) api.DEBUG = true;

			// Next deal with MIBs
    MibModule m1, m2;
    if ( (values[2] !=null)||(values[3] !=null) ) try { // Open the MIB module

	System.out.println("Loading MIB files: " + values[2] +" "+ values[3]);
	if (values[2] != null) m1 = new MibModule(values[2], api,api.DEBUG);
	if (values[3] != null) m2 = new MibModule(values[3], api,api.DEBUG);
	System.out.println("Done.\n");

    } catch(MibException e) { System.err.println(e); }
      catch(java.io.IOException e) { System.err.println(e); }


    if (opt.remArgs.length<2) opt.usage_error();

           	// Open session and set remote host & port if needed,
    SnmpSession session = new SnmpSession(api);

    session.peername = opt.remArgs[0];
    if (values[1] != null) session.community = values[1];

    try {
      if (values[4] != null) session.remote_port = Integer.parseInt(values[4]);
      if (values[5] != null) session.retries = Integer.parseInt(values[5]);
      if (values[6] != null) session.timeout = Integer.parseInt(values[6]);
    } catch (NumberFormatException ex) {
	System.err.println("Invalid Integer Arg");
    }


		// Build get request PDU
    SnmpPDU npdu = null, pdu = new SnmpPDU(api);
    pdu.command = api.GETNEXT_REQ_MSG;

    long rootoid[]=null;
    for (int i=1;i<2;i++) { // add OIDs - only one allowed for snmpwalk
      SnmpOID oid = new SnmpOID(opt.remArgs[i], api);
      if ((rootoid = (long[])oid.toValue()) == null)
	System.err.println("Invalid OID argument: " + opt.remArgs[i]);
      else pdu.addNull(oid);
    }

    try { // first open the session
	session.open();
		// Send PDU
    } catch (SnmpException e) {
	System.err.println("Open Session: "+e.getMessage());
    }

	// loop for each PDU in the walk
    while (true) { // until received OID isn't in sub-tree

      try { npdu = session.syncSend(pdu); } // Send PDU 
      catch (SnmpException e) {
	System.err.println("Sending PDU: "+e.getMessage());
        System.exit(1);
      }

      if (npdu == null) {  // timeout
          System.out.println("Request timed out to: " + opt.remArgs[0] );
          System.exit(1);
      }

      if (api.DEBUG) System.out.println("Response received from " 
		+npdu.address+", community: " + npdu.community);

      if (npdu.errstat != 0) {
         System.out.println("Error Indication in response: " +
                            SnmpException.exceptionString((byte)npdu.errstat)
			    +"\nErrindex: " + npdu.errindex);
	 System.exit(1);
      }

     if (!isInSubTree(rootoid,npdu)) break; // stop if outside sub-tree

      System.out.println(npdu.printVarBinds());

      // Put in the returned OID for the getnext
      SnmpVarBind varbind = (SnmpVarBind) pdu.variables.firstElement();
      SnmpVarBind nvarbind = (SnmpVarBind) npdu.variables.firstElement();

      if (nvarbind == null) System.exit(1);

      varbind.oid = new SnmpOID(nvarbind.oid.toString(), api);

      pdu.command = api.GETNEXT_REQ_MSG;
      pdu.reqid = 0; // Make it 0 so this will be set by the api 

    } // end of while true

    System.exit(0);

  } // end of main


/** check if first varbind oid has rootoid as an ancestor */
  static boolean isInSubTree(long[] rootoid, SnmpPDU pdu) {

    SnmpVarBind varbind = (SnmpVarBind) pdu.variables.firstElement();
    if (varbind == null) return false;

    long oid[] = (long[]) varbind.oid.toValue();
    if (oid == null) return false;

    if (oid.length < rootoid.length) return false;

    for (int i=0;i<rootoid.length;i++)
	if (oid[i] != rootoid[i]) return false;

    return true;
  }
}
