package MCSocket;

// ad ogni nuova connessione viene creato un nuovo thread che gestisce
// il singolo socket
// all'arrivo dei pacchetti viene insrito nella coda relativa allo specifico
// socket

/**
 *  This class manages all active connection returning the
 *  socket on which is listening the upper layer
 *
 *  @author   Michele Malgeri
 *  @version  %I%, %G%
 *  @since    MAP2.0
 *
 */

import java.io.*;
import java.net.*;
import java.lang.Thread;
import java.util.*;

public class queueManager extends Thread {
  private Queue q;
  private Queue cq;
  private connectionManager cm;
  private Hashtable ht = new Hashtable();
  private MSocket socket;
    
  /**
   *  Empty constructor
   *
   */
  queueManager() {
  }
  
  /**
   *  Constructor
   *
   *  @param q    the queue containing packet to be served
   *  @param cm   it contains the data regarding the active
   *              connections
   *
   */
  queueManager(Queue q, connectionManager cm) {
    this.q = q;
    this.cm = cm;
    cq = new Queue();
  }
   
  /**
   *  Constructor
   *
   *  @param q    the queue containing packet to be served
   *  @param cq   the queue containing sockets dealing with new connection
   *  @param cm   it contains the data regarding the active
   *              connections
   *
   */
  queueManager(Queue q, Queue cq, connectionManager cm) {
    this.q = q;
    this.cq = cq;
    this.cm = cm;
  }
   
  /**
   *  Constructor
   *
   *  @param MSocket    the queue containing packet to be served
   *  @param cm   it contains the data regarding the active
   *              connections
   *
   *  @see 
   
   */
  queueManager(MSocket socket, connectionManager cm) {
    this.socket = socket;
    this.cm = cm;
  }
   

  /**
   *  This function enqueue the socket associated with a new connection
   *
   *  @see  put
   */
  public MSocket getNewSocket() {
    return (MSocket) cq.remove();
  }

  /**
   *  This function returns the socket associated with a new connection
   *
   *  @param  socket  socket dealing with a new connection
   *
   *  @see  get
   */
  public void putNewSocket(MSocket socket) {
    cq.insert(socket);    
  }
  
  
  public void run() {
    DatagramPacket  dp;
    boolean open = true;      
    //
    // dovrebbe rimanere in attesa che qualche pacchetto sia disponibile, quindi 
    // lo dovrebbe inviare al giusto output stream
    //

    try {
      while(open) {
        if(config.DEBUG) System.out.println("DEBUG (queueManager): waiting data");
          dp = (DatagramPacket) socket.getQueue().remove();

        //
        // first byte contains information about the type of packet or its 
        // sequence number if number is greater than 0
        //

        if(dp.getData()[0] == 0) {
          if(config.DEBUG) System.out.println("DEBUG (queueManager): info packet received "
                                              + (int) dp.getData()[1]); 
          //
          // When packet contains information second byte contains type of packet
          //
          switch(dp.getData()[1]) {
            case 0:   // meaning connection request
              System.err.println("ERROR (queueManager): Connection request can't be managed here!");
              break; 
            case 1:   // meaning close connection
              if(config.DEBUG) System.out.println("DEBUG (queueManager): connection closed " +
                                                    dp.getAddress() +
                                                    " port = " +
                                                    dp.getPort());

              
              try {
                cm.remove(dp.getAddress(), dp.getPort());
              } catch(Exception e) {
                  System.err.println("ERROR (queueManager): can't remove connection id");
                  break;
                }
              open = false;
              break;
            case 2:   // meaning continues (stream is longer than 254 packets)
              if(config.DEBUG) System.out.println("DEBUG (queueManager): continue");
              break;
            default:  // unknown type
              if(config.DEBUG) System.out.println("DEBUG (queueManager): unknown type");
              break;
          }
        }
        else {    // data packet received
          if(config.DEBUG) System.out.println("DEBUG (queueManager): datapacket address = " +
                                              dp.getAddress() +
                                              " port = " +
                                              dp.getPort() +
                                              " #" +
                                              dp.getData()[0]);
                                
                
          try {
            OutputStream os = socket.getOutputStream();
            os.write(dp.getData(), 1, dp.getLength() - 1);
          } catch(IOException e) {
              System.err.println("ERROR (queueManager): error writing");
              break;
            }
          if(config.DEBUG) System.out.println("DEBUG (queuemanager): datapacket #"
                                            + dp.getData()[0]
                                            + " added "
                                            + dp.getLength() );
        }
      }
    } catch(Exception e) {
        System.err.println("ERROR (QueueManager): Generic exception received");
      }
    if(config.DEBUG) System.out.println("DEBUG (queueManager): exiting");
  }
}