package MCSocket;

import java.io.*;
import java.net.*;
import java.util.*;

public class MSocket extends Thread {
  private static final int DEBUG = 1;
  private PipedInputStream  pis;
  private PipedOutputStream pos;

  private int port;
  private InetAddress address;
  private int localPort;
  private InetAddress localAddress ;
  
  private byte[] buf = new byte[1024];
  private DatagramPacket dp = new DatagramPacket(buf, buf.length);
  private MulticastSocket ds;

  private Queue q = new Queue();
    
  public MSocket() throws IOException {
    //this.localAddress = address;
    //this.localPort = port;
    //this.address = 0;
    //this.port = 0;

    if(config.DEBUG)
      System.out.println("DEBUG (MSocket): new socket created");
    byte[] buf = new byte[1024];

    buf[0] = 0;
    buf[1] = 1;
    DatagramPacket dp = new DatagramPacket(buf, buf.length, InetAddress.getByName("0.0.0.0"), 0);
    pis = new PipedInputStream();
    pos = new PipedOutputStream(pis);
  }
  
  public MSocket(InetAddress address, int port) throws IOException {
    // 
    // cerca una porta libera 
    if(config.DEBUG) 
      System.out.println("DEBUG (MSocket): initializing");
    try {
      ds = new MulticastSocket(localPort);
    } catch(SocketException e) {
        System.err.println("ERROR (MSocket): can't open local socket");
      }
        
    // Local address
    this.localAddress = InetAddress.getByName("localhost");;
    this.localPort = ds.getLocalPort();

    // Remote Address
    this.address = address;
    this.port = port;

    pis = new PipedInputStream();
    pos = new PipedOutputStream(pis);
            
    if(config.DEBUG) 
      System.out.println("DEBUG (MSocket): local port is " + ds.getLocalPort());

    start();
  }

  public MSocket(InetAddress address, int port, 
          InetAddress localAddress, int localPort) throws IOException {

    this.localAddress = address;
    this.localPort = localPort;

    this.address = address;
    this.port = port;
    
    pis = new PipedInputStream();
    pos = new PipedOutputStream(pis);
            
    // Bisogna cercare una port libera
    try {
      ds = new MulticastSocket(localPort);
    } catch(SocketException e) {
        System.err.println("can't open local socket on port " + localPort);
      }

    start();
  }
  
  public synchronized void close() throws IOException {
    pos.flush();
    //pis.close();
    pos.close();
    q.insert(dp);    
    //ds.close();
  }
  
  public String toString() {
    return "MSocket: " + pis;
  }
  
  public InputStream getInputStream() {
    return (InputStream) pis;
  }
  
  public OutputStream getOutputStream() {
    return (OutputStream) pos;
  }
  
  public Queue getQueue() {
    return q;
  }
  
  public void run() {
    DataInputStream dis = new DataInputStream(pis);
    DatagramPacket dp;
    
    int counter = 1;
      
    // send initialization to remote host
    buf[0] = 0;   // meaning management packet
    buf[1] = 0;   // meaning open connection
    try {
      ds.send(new DatagramPacket(buf, buf.length, address, port));
      if(config.DEBUG) 
        System.out.println("DEBUG (MSocket):opening connection with " 
                            + address
                            + ":"
                            + port);
    } catch(IOException e) {
        System.err.println("IOException");
      }
                              
    try {           
      try {
        while(true) {
          //try {
          dis.readFully(buf, 1, 1023);
          //} catch (IOException r) { 
          //  System.out.println("pac");}
          buf[0] = (byte) counter++;
          ds.send(new DatagramPacket(buf, buf.length, address, port));
          if(DEBUG == 1)
            System.out.println("packet #"
                                + (counter - 1)
                                + " to node "
                                + address
                                + ":"
                                + port);
        } 
      } catch(EOFException eof) {
          if(DEBUG == 1)
            System.out.println("DEBUG (MSocket): punto 1");

          buf[0] = (byte) counter++;
          dp = new DatagramPacket(buf, buf.length, address, port);
          ds.send(dp);

      
          if(config.DEBUG)
            System.out.println("DEBUG (MSocket): last packet #"
                                + (counter - 1)
                                + " to node "
                                + address
                                + ":"
                                + port);
        
          // messaggio di fine connessione

          byte[] nbuf = new byte[1024];
          nbuf[0] = 0;   // meaning management packet
          nbuf[1] = 1;   // meaning close connection
          ds.send(new DatagramPacket(nbuf, nbuf.length, address, port));
          if(config.DEBUG)
            System.out.println("DEBUG (MSocket): closed connection with "
                                + address
                                + ":"
                                + port);
        
          //pis.close();
          //pos.close();
          ds.close();
        }
    } catch(IOException e) {
        System.err.println("ERROR (MSocket): IOException");
      }
    if(DEBUG == 1)
      System.out.println("fine gestione socket");
  }
}
